# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

einsum_impl_fast <- function(lengths_vec, array_vars_list, not_result_vars_vec, result_vars_vec, arrays) {
    .Call(`_einsum_einsum_impl_fast`, lengths_vec, array_vars_list, not_result_vars_vec, result_vars_vec, arrays)
}

